@@ -8,13 +8,12 @@ sub labels {
     # "packing list" variable which specifies the list of packer attributes.
 
     my($demo) = @_;
-    my $demo_widget = $MW->WidgetDemo(
+    $TOP = $MW->WidgetDemo(
         -name     => $demo,
         -text     => 'Five labels are displayed below: three textual ones on the left, and an image label and a text label on the right.  Labels are pretty boring because you can\'t do anything with them.',
         -title    => 'Label Demonstration',
         -iconname => 'label',
     );
-    $TOP = $demo_widget->Top;	# get geometry master
 
     my(@pl) = qw/-side left -expand yes -padx 10 -pady 10 -fill both/;
     my $left = $TOP->Frame->pack(@pl);
@@ -23,7 +22,7 @@ sub labels {
     @pl = qw/-side top -expand yes -pady 2 -anchor w/;
     my $left_l1 = $left->Label(-text => 'First label')->pack(@pl);
     my $left_l2 = $left->Label(
-        -text   => 'Second label, raised just for fun', 
+        -text   => 'Second label, raised just for fun',
         -relief => 'raised',
     )->pack(@pl);
     my $left_l3 = $left->Label(
