@@ -23,7 +23,7 @@ sub msgBox {
     $lsep->pack(qw/-side top -fill x -expand no/);
 
     my $iconvar = 'info';
-    foreach my $i (qw/error hourglass info question questhead warning/) {
+    foreach my $i (qw/error info question warning/) {
 	$left->Radiobutton(-text => $i, -variable => \$iconvar, -value => $i,
 			   qw/-width 16 -anchor w -relief flat/)->pack(
 			   qw/-side top -pady 2 -anchor w -fill x/);
@@ -42,10 +42,31 @@ sub msgBox {
 			    qw/-side top -pady 2 -anchor w -fill x/);
     }
 
-    my $show = $TOP->Button(-text => "Message Box", 
+    {
+	my $cbf = $TOP->Frame->pack(-fill => 'x', -padx => '.5c', -pady => 3);
+	my $fd;
+	my $fdb = $cbf->Radiobutton
+	    (-text => 'Tk::DialogBox',
+	     -variable => \$fd,
+	     -value => 'DialogBox',
+	     -command => sub { local($^W) = 0;
+			       *Tk::tk_messageBox = sub{ Tk::MessageBox('tk_messageBox', @_) };
+			   })->pack(-side => 'left');
+	$cbf->Radiobutton
+	    (-text => 'Tk::MsgBox',
+	     -variable => \$fd,
+	     -value => 'MsgBox',
+	     -command => sub { local($^W) = 0;
+			       require Tk::MsgBox;
+			       Tk::MsgBox->import('as_default');
+			   })->pack(-side => 'left');
+	$fdb->invoke;
+    }
+
+    my $show = $TOP->Button(-text => "Message Box",
 			    -command => [\&show_messageBox, \$iconvar, \$typevar]);
     $show->pack;
-    
+
 } # end msgBox
 
 sub show_messageBox {
