@@ -8,22 +8,21 @@ sub vscale {
     # Create a top-level window that displays a vertical scale.
 
     my($demo) = @_;
-    my $demo_widget = $MW->WidgetDemo(
+    $TOP = $MW->WidgetDemo(
         -name     => $demo,
         -text     => 'An arrow and a vertical scale are displayed below.  If you click or drag mouse button 1 in the scale, you can change the size of the arrow.',
         -title    => 'Vertical Scale Demonstration',
         -iconname => 'vscale',
     );
-    $TOP = $demo_widget->Top;	# get geometry master
 
     my $frame = $TOP->Frame(-borderwidth => 10)->pack;
 
     my $canvas = $frame->Canvas(
         qw/-width 50 -height 50 -borderwidth 0 -highlightthickness 0/);
-    $canvas->create(qw/polygon 0 0 1 1 2 2 -fill SeaGreen3 -tags poly/);
-    $canvas->create(qw/line 0 0 1 1 2 2 0 0 -fill black -tags line/);
+    $canvas->createPolygon(qw/0 0 1 1 2 2 -fill SeaGreen3 -tags poly/);
+    $canvas->createLine(qw/0 0 1 1 2 2 0 0 -fill black -tags line/);
 
-    my $scale = $frame->Scale(qw/-orient vertical -length 284 -from 0 
+    my $scale = $frame->Scale(qw/-orient vertical -length 284 -from 0
         -to 250 -tickinterval 50 -command/ => [\&vscale_height, $canvas]);
     $scale->set(75);
 
