@@ -13,6 +13,10 @@
 #include <langinfo.h>
 #endif
 
+#define NEED_utf8_to_uvchr_buf
+#define NEED_my_strnlen
+#include "ppport.h"
+
 #define U8 U8
 #include "tkGlue.def"
 
@@ -24,15 +28,15 @@
 #include "pTk/tkWinInt.h"
 #endif
 
-
 #ifdef SvUTF8
 
-#ifndef utf8_to_uv
-#define utf8_to_uv utf8_to_uvchr
-#endif
 
-#ifndef UTF8_MAXBYTES_CASE
-#define UTF8_MAXBYTES_CASE UTF8_MAXLEN_UCLC
+/* Workaround for immediate crash with perl 5.19.9+ and without XFT
+ * See https://rt.cpan.org/Ticket/Display.html?id=96543 and
+ * https://rt.perl.org/Ticket/Display.html?id=120626
+ */
+#if !defined(USE_XFT_FONTS) && PERL_VERSION > 19 || (PERL_VERSION == 19 && PERL_SUBVERSION >= 9)
+#define NEED_PUSHSTACK_HACK
 #endif
 
 /* -------------------------------------------------------------------------- */
@@ -55,7 +59,7 @@ Tcl_UniCharToUpper(int ch)
  dTHX;
  U8 tmpbuf[UTF8_MAXBYTES_CASE+1];
  STRLEN len;
- return Perl_to_uni_upper(aTHX_ ch, tmpbuf, &len);
+ return toUPPER_uvchr(ch, tmpbuf, &len);
 }
 
 Tcl_UniChar
@@ -64,35 +68,35 @@ Tcl_UniCharToLower(int ch)
  dTHX;
  U8 tmpbuf[UTF8_MAXBYTES_CASE+1];
  STRLEN len;
- return Perl_to_uni_lower(aTHX_ ch, tmpbuf, &len);
+ return toLOWER_uvchr(ch, tmpbuf, &len);
 }
 
 int
 Tcl_UniCharIsAlpha(int ch)
 {
  dTHX;
- return Perl_is_uni_alpha(aTHX_ ch);
+ return isALPHA_uvchr(ch);
 }
 
 int
 Tcl_UniCharIsWordChar(int ch)
 {
  dTHX;
- return Perl_is_uni_alnum(aTHX_ ch);
+ return isWORDCHAR_uvchr(ch);
 }
 
 int
 Tcl_UniCharIsSpace(int ch)
 {
  dTHX;
- return Perl_is_uni_space(aTHX_ ch);
+ return isSPACE_uvchr(ch);
 }
 
 int
 Tcl_UniCharIsUpper(int ch)
 {
  dTHX;
- return Perl_is_uni_upper(aTHX_ ch);
+ return isUPPER_uvchr(ch);
 }
 
 int
@@ -127,7 +131,7 @@ Tcl_UtfPrev (CONST char * src,CONST char * start)
  dTHX;
  U8 *s = (U8 *) src;
  if (src > start)
-  return (CONST char *) Perl_utf8_hop(aTHX_ s,-1);
+  return (CONST char *) utf8_hop(s,-1);
  else
   return (CONST char *) s;
 }
@@ -137,20 +141,15 @@ Tcl_UtfAtIndex (CONST char * src, int index)
 {
  dTHX;
  U8 *s = (U8 *) src;
- return (CONST char*)Perl_utf8_hop(aTHX_ s,index);
+ return (CONST char*)utf8_hop(s,index);
 }
 
 int
 Tcl_UtfToUniChar (CONST char * src,Tcl_UniChar * chPtr)
 {
  dTHX;
-#if defined(utf8_to_uvchr)
  STRLEN len;
- *chPtr = utf8_to_uv((U8 *)src,&len);
-#else
- I32 len;
- *chPtr = utf8_to_uv((U8 *)src,&len);
-#endif
+ *chPtr = utf8_to_uvchr_buf((U8 *)src, src + UTF8_CHK_SKIP(src), &len);
  return len;
 }
 
@@ -265,7 +264,7 @@ Tcl_UtfToLower (char * src)
  while (*s)
   {
    STRLEN len;
-   Perl_to_utf8_lower(aTHX_ s, d, &len );
+   toLOWER_utf8_safe(s, s + UTF8_CHK_SKIP(s), d, &len );
    d += len;
    s += len;
   }
@@ -282,7 +281,7 @@ Tcl_UtfToUpper(char * src)
  while (*s)
   {
    STRLEN len;
-   Perl_to_utf8_upper(aTHX_ s, d, &len );
+   toUPPER_utf8_safe(s, s + UTF8_CHK_SKIP(s), d, &len );
    d += len;
    s += len;
   }
@@ -298,25 +297,29 @@ Tcl_UtfToUpper(char * src)
 Tcl_UniChar
 Tcl_UniCharToUpper(int ch)
 {
- return toupper(ch);
+         /* Only the ASCII characters and those in E0..FE have upper case
+          * values that are still in range */
+ return ((isLOWER_L1(ch) && (isASCII(ch) || (ch >= 0xE0 && ch <= 0xFE)))
+         ? ch - 0x20
+         : ch);
 }
 
 Tcl_UniChar
 Tcl_UniCharToLower(int ch)
 {
- return tolower(ch);
+ return ((! isUPPER_L1(ch)) ? ch : ch + 0x20)
 }
 
 int
 Tcl_UniCharIsAlpha(int ch)
 {
- return isalpha(ch);
+ return isALPHA_L1(ch);
 }
 
 int
 Tcl_UniCharIsUpper(int ch)
 {
- return isupper(ch);
+ return isUPPER_L1(ch);
 }
 
 int
@@ -334,7 +337,7 @@ Tcl_UtfToLower (char * src)
  int n = 0;
  while (*s)
   {
-   *s = tolower(UCHAR(*s));
+   *s = Tcl_UniCharToLower(UCHAR(*s));
    s++;
   }
  *s = '\0';
@@ -348,7 +351,7 @@ Tcl_UtfToUpper(char * src)
  int n = 0;
  while (*s)
   {
-   *s = toupper(UCHAR(*s));
+   *s = Tcl_UniCharToUpper(UCHAR(*s));
    s++;
   }
  *s = '\0';
@@ -593,6 +596,9 @@ Tcl_GetEncoding (Tcl_Interp * interp, CONST char * name)
    dSP;
    ENTER;
    SAVETMPS;
+#ifdef NEED_PUSHSTACK_HACK
+   PUSHSTACKi(PERLSI_REQUIRE);
+#endif
    PUSHMARK(sp);
    XPUSHs(sv_2mortal(newSVpv("Tk",0)));
    XPUSHs(nmsv);
@@ -604,6 +610,9 @@ Tcl_GetEncoding (Tcl_Interp * interp, CONST char * name)
    he = hv_store_ent(encodings,nmsv,newSVsv(sv),0);
    if (0 && !SvOK(sv))
     warn("Cannot find '%s'",name);
+#ifdef NEED_PUSHSTACK_HACK
+   POPSTACK;
+#endif
    FREETMPS;
    LEAVE;
   }
