@@ -1179,7 +1179,8 @@ MovePointer2(sourcePtr, destPtr, mode, leaveEvents, enterEvents)
     event.xcrossing.display = winPtr->display;
     event.xcrossing.root = RootWindow(winPtr->display,
 	    winPtr->screenNum);
-    event.xcrossing.time = TkCurrentTime(winPtr->dispPtr);
+    event.xcrossing.time = TkCurrentTime(winPtr->dispPtr, False);
+    /* FALSE because we want a real time (even an old one) not CurrentTime */
     XQueryPointer(winPtr->display, winPtr->window, &dummy1, &dummy2,
 	    &event.xcrossing.x_root, &event.xcrossing.y_root,
 	    &dummy3, &dummy4, &event.xcrossing.state);
