@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclFileName.c,v 1.37 2002/06/05 11:59:33 das Exp $
+ * RCS: @(#) $Id: tclFileName.c,v 1.40.2.5 2003/10/06 09:49:19 vincentdarley Exp $
  */
 
 #include "tclInt.h"
@@ -243,9 +243,51 @@ ExtractWinRoot(path, resultPtr, offset, typePtr)
 	    return tail;
 	}
     } else {
-	*typePtr = TCL_PATH_RELATIVE;
-	return path;
+	int abs = 0;
+	if (path[0] == 'c' && path[1] == 'o') {
+	    if (path[2] == 'm' && path[3] >= '1' && path[3] <= '9') {
+		/* May have match for 'com[1-9]:?', which is a serial port */
+	        if (path[4] == '\0') {
+	            abs = 4;
+	        } else if (path [4] == ':' && path[5] == '\0') {
+		    abs = 5;
+	        }
+	    } else if (path[2] == 'n' && path[3] == '\0') {
+		/* Have match for 'con' */
+		abs = 3;
+	    }
+	} else if (path[0] == 'l' && path[1] == 'p' && path[2] == 't') {
+	    if (path[3] >= '1' && path[3] <= '9') {
+		/* May have match for 'lpt[1-9]:?' */
+		if (path[4] == '\0') {
+		    abs = 4;
+		} else if (path [4] == ':' && path[5] == '\0') {
+		    abs = 5;
+		}
+	    }
+	} else if (path[0] == 'p' && path[1] == 'r'
+		   && path[2] == 'n' && path[3] == '\0') {
+	    /* Have match for 'prn' */
+	    abs = 3;
+	} else if (path[0] == 'n' && path[1] == 'u'
+		   && path[2] == 'l' && path[3] == '\0') {
+	    /* Have match for 'nul' */
+	    abs = 3;
+	} else if (path[0] == 'a' && path[1] == 'u'
+		   && path[2] == 'x' && path[3] == '\0') {
+	    /* Have match for 'aux' */
+	    abs = 3;
+	}
+	if (abs != 0) {
+	    *typePtr = TCL_PATH_ABSOLUTE;
+	    Tcl_DStringSetLength(resultPtr, offset);
+	    Tcl_DStringAppend(resultPtr, path, abs);
+	    return path + abs;
+	}
     }
+    /* Anything else is treated as relative */
+    *typePtr = TCL_PATH_RELATIVE;
+    return path;
 }
 
 /*
@@ -1338,17 +1380,19 @@ Tcl_TranslateFileName(interp, name, bufferPtr)
 				 * with name after tilde substitution. */
 {
     Tcl_Obj *path = Tcl_NewStringObj(name, -1);
-    CONST char *result;
+    Tcl_Obj *transPtr;
 
     Tcl_IncrRefCount(path);
-    result = Tcl_FSGetTranslatedStringPath(interp, path);
-    if (result == NULL) {
+    transPtr = Tcl_FSGetTranslatedPath(interp, path);
+    if (transPtr == NULL) {
 	Tcl_DecrRefCount(path);
 	return NULL;
     }
+
     Tcl_DStringInit(bufferPtr);
-    Tcl_DStringAppend(bufferPtr, result, -1);
+    Tcl_DStringAppend(bufferPtr, Tcl_GetString(transPtr), -1);
     Tcl_DecrRefCount(path);
+    Tcl_DecrRefCount(transPtr);
 
     /*
      * Convert forward slashes to backslashes in Windows paths because
@@ -1619,7 +1663,6 @@ Tcl_GlobObjCmd(dummy, interp, objc, objv)
 	    case GLOB_LAST:				/* -- */
 	        i++;
 		goto endOfForLoop;
-		break;
 	}
     }
     endOfForLoop:
@@ -2269,8 +2312,19 @@ TclDoGlob(interp, separators, headPtr, tail, types)
     count = 0;
     name = tail;
     for (; *tail != '\0'; tail++) {
-	if ((*tail == '\\') && (strchr(separators, tail[1]) != NULL)) {
-	    tail++;
+	if (*tail == '\\') {
+	    /*
+	     * If the first character is escaped, either we have a directory
+	     * separator, or we have any other character.  In the latter case
+	     * the rest of tail is a pattern, and we must break from the loop.
+	     * This is particularly important on Windows where '\' is both
+	     * the escaping character and a directory separator.
+	     */
+	    if (strchr(separators, tail[1]) != NULL) {
+		tail++;
+	    } else {
+		break;
+	    }
 	} else if (strchr(separators, *tail) == NULL) {
 	    break;
 	}
@@ -2517,26 +2571,34 @@ TclDoGlob(interp, separators, headPtr, tail, types)
 	 */
 
 	Tcl_Obj *nameObj;
-	/* Used to deal with one special case pertinent to MacOS */
-	int macSpecialCase = 0;
 
 	switch (tclPlatform) {
 	    case TCL_PLATFORM_MAC: {
 		if (strchr(Tcl_DStringValue(headPtr), ':') == NULL) {
 		    Tcl_DStringAppend(headPtr, ":", 1);
 		}
-		macSpecialCase = 1;
 		break;
 	    }
 	    case TCL_PLATFORM_WINDOWS: {
 		if (Tcl_DStringLength(headPtr) == 0) {
 		    if (((*name == '\\') && (name[1] == '/' || name[1] == '\\'))
 			    || (*name == '/')) {
-			Tcl_DStringAppend(headPtr, "\\", 1);
+			Tcl_DStringAppend(headPtr, "/", 1);
 		    } else {
 			Tcl_DStringAppend(headPtr, ".", 1);
 		    }
 		}
+#if defined(__CYGWIN__) && defined(__WIN32__)
+		{
+		extern int cygwin_conv_to_win32_path
+		    _ANSI_ARGS_((CONST char *, char *));
+		char winbuf[MAX_PATH+1];
+
+		cygwin_conv_to_win32_path(Tcl_DStringValue(headPtr), winbuf);
+		Tcl_DStringFree(headPtr);
+		Tcl_DStringAppend(headPtr, winbuf, -1);
+		}
+#endif /* __CYGWIN__ && __WIN32__ */
 		/*
 		 * Convert to forward slashes.  This is required to pass
 		 * some Tcl tests.  We should probably remove the conversions
