@@ -1,16 +1,16 @@
 /*
  * tclWinNotify.c --
  *
- *	This file contains Windows-specific procedures for the notifier,
- *	which is the lowest-level part of the Tcl event loop.  This file
- *	works together with ../generic/tclNotify.c.
+ *      This file contains Windows-specific procedures for the notifier,
+ *      which is the lowest-level part of the Tcl event loop.  This file
+ *      works together with ../generic/tclNotify.c.
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinNotify.c,v 1.10 2001/12/14 10:32:47 dkf Exp $
+ * RCS: @(#) $Id: tclWinNotify.c,v 1.11.2.1 2003/03/21 03:24:09 dgp Exp $
  */
 
 #include <windows.h>
@@ -23,9 +23,9 @@
  * The follwing static indicates whether this module has been initialized.
  */
 
-#define INTERVAL_TIMER 1	/* Handle of interval timer. */
+#define INTERVAL_TIMER 1        /* Handle of interval timer. */
 
-#define WM_WAKEUP WM_USER	/* Message that is send by
+#define WM_WAKEUP WM_USER       /* Message that is send by
 				 * Tcl_AlertNotifier. */
 /*
  * The following static structure contains the state information for the
@@ -34,21 +34,23 @@
  */
 
 typedef struct ThreadSpecificData {
-    CRITICAL_SECTION crit;	/* Monitor for this notifier. */
-    DWORD thread;		/* Identifier for thread associated with this
+    CRITICAL_SECTION crit;      /* Monitor for this notifier. */
+    DWORD thread;               /* Identifier for thread associated with this
 				 * notifier. */
-    HANDLE event;		/* Event object used to wake up the notifier
+    HANDLE event;               /* Event object used to wake up the notifier
 				 * thread. */
-    int pending;		/* Alert message pending, this field is
+    int pending;                /* Alert message pending, this field is
 				 * locked by the notifierMutex. */
-    HWND hwnd;			/* Messaging window. */
-    int timeout;		/* Current timeout value. */
-    int timerActive;		/* 1 if interval timer is running. */
+    HWND hwnd;                  /* Messaging window. */
+    int timeout;                /* Current timeout value. */
+    int timerActive;            /* 1 if interval timer is running. */
 } ThreadSpecificData;
 
 static Tcl_ThreadDataKey dataKey;
 
 extern TclStubs tclStubs;
+extern Tcl_NotifierProcs tclOriginalNotifier;
+
 /*
  * The following static indicates the number of threads that have
  * initialized notifiers.  It controls the lifetime of the TclNotifier
@@ -64,7 +66,7 @@ TCL_DECLARE_MUTEX(notifierMutex)
  * Static routines defined in this file.
  */
 
-static LRESULT CALLBACK	NotifierProc(HWND hwnd, UINT message,
+static LRESULT CALLBACK NotifierProc(HWND hwnd, UINT message,
 			    WPARAM wParam, LPARAM lParam);
 
 
@@ -73,13 +75,13 @@ static LRESULT CALLBACK	NotifierProc(HWND hwnd, UINT message,
  *
  * Tcl_InitNotifier --
  *
- *	Initializes the platform specific notifier state.
+ *      Initializes the platform specific notifier state.
  *
  * Results:
- *	Returns a handle to the notifier state for this thread..
+ *      Returns a handle to the notifier state for this thread..
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
@@ -133,21 +135,21 @@ Tcl_InitNotifier()
  *
  * Tcl_FinalizeNotifier --
  *
- *	This function is called to cleanup the notifier state before
- *	a thread is terminated.
+ *      This function is called to cleanup the notifier state before
+ *      a thread is terminated.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	May dispose of the notifier window and class.
+ *      May dispose of the notifier window and class.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_FinalizeNotifier(clientData)
-    ClientData clientData;	/* Pointer to notifier data. */
+    ClientData clientData;      /* Pointer to notifier data. */
 {
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *) clientData;
 
@@ -195,27 +197,27 @@ Tcl_FinalizeNotifier(clientData)
  *
  * Tcl_AlertNotifier --
  *
- *	Wake up the specified notifier from any thread. This routine
- *	is called by the platform independent notifier code whenever
- *	the Tcl_ThreadAlert routine is called.  This routine is
- *	guaranteed not to be called on a given notifier after
- *	Tcl_FinalizeNotifier is called for that notifier.  This routine
- *	is typically called from a thread other than the notifier's
- *	thread.
+ *      Wake up the specified notifier from any thread. This routine
+ *      is called by the platform independent notifier code whenever
+ *      the Tcl_ThreadAlert routine is called.  This routine is
+ *      guaranteed not to be called on a given notifier after
+ *      Tcl_FinalizeNotifier is called for that notifier.  This routine
+ *      is typically called from a thread other than the notifier's
+ *      thread.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Sends a message to the messaging window for the notifier
- *	if there isn't already one pending.
+ *      Sends a message to the messaging window for the notifier
+ *      if there isn't already one pending.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_AlertNotifier(clientData)
-    ClientData clientData;	/* Pointer to thread data. */
+    ClientData clientData;      /* Pointer to thread data. */
 {
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *) clientData;
 
@@ -246,22 +248,22 @@ Tcl_AlertNotifier(clientData)
  *
  * Tcl_SetTimer --
  *
- *	This procedure sets the current notifier timer value.  The
- *	notifier will ensure that Tcl_ServiceAll() is called after
- *	the specified interval, even if no events have occurred.
+ *      This procedure sets the current notifier timer value.  The
+ *      notifier will ensure that Tcl_ServiceAll() is called after
+ *      the specified interval, even if no events have occurred.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Replaces any previous timer.
+ *      Replaces any previous timer.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_SetTimer(
-    Tcl_Time *timePtr)		/* Maximum block time, or NULL. */
+    Tcl_Time *timePtr)          /* Maximum block time, or NULL. */
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
     UINT timeout;
@@ -272,7 +274,7 @@ Tcl_SetTimer(
      */
 
 #ifndef _LANG
-    if (tclStubs.tcl_SetTimer != Tcl_SetTimer) {
+    if (tclStubs.tcl_SetTimer != tclOriginalNotifier.setTimerProc) {
 	tclStubs.tcl_SetTimer(timePtr);
 	return;
     }
@@ -318,21 +320,21 @@ Tcl_SetTimer(
  *
  * Tcl_ServiceModeHook --
  *
- *	This function is invoked whenever the service mode changes.
+ *      This function is invoked whenever the service mode changes.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	If this is the first time the notifier is set into
- *	TCL_SERVICE_ALL, then the communication window is created.
+ *      If this is the first time the notifier is set into
+ *      TCL_SERVICE_ALL, then the communication window is created.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_ServiceModeHook(mode)
-    int mode;			/* Either TCL_SERVICE_ALL, or
+    int mode;                   /* Either TCL_SERVICE_ALL, or
 				 * TCL_SERVICE_NONE. */
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
@@ -366,16 +368,16 @@ Tcl_ServiceModeHook(mode)
  *
  * NotifierProc --
  *
- *	This procedure is invoked by Windows to process events on
- *	the notifier window.  Messages will be sent to this window
- *	in response to external timer events or calls to
- *	TclpAlertTsdPtr->
+ *      This procedure is invoked by Windows to process events on
+ *      the notifier window.  Messages will be sent to this window
+ *      in response to external timer events or calls to
+ *      TclpAlertTsdPtr->
  *
  * Results:
- *	A standard windows result.
+ *      A standard windows result.
  *
  * Side effects:
- *	Services any pending events.
+ *      Services any pending events.
  *
  *----------------------------------------------------------------------
  */
@@ -410,24 +412,24 @@ NotifierProc(
  *
  * Tcl_WaitForEvent --
  *
- *	This function is called by Tcl_DoOneEvent to wait for new
- *	events on the message queue.  If the block time is 0, then
- *	Tcl_WaitForEvent just polls the event queue without blocking.
+ *      This function is called by Tcl_DoOneEvent to wait for new
+ *      events on the message queue.  If the block time is 0, then
+ *      Tcl_WaitForEvent just polls the event queue without blocking.
  *
  * Results:
- *	Returns -1 if a WM_QUIT message is detected, returns 1 if
- *	a message was dispatched, otherwise returns 0.
+ *      Returns -1 if a WM_QUIT message is detected, returns 1 if
+ *      a message was dispatched, otherwise returns 0.
  *
  * Side effects:
- *	Dispatches a message to a window procedure, which could do
- *	anything.
+ *      Dispatches a message to a window procedure, which could do
+ *      anything.
  *
  *----------------------------------------------------------------------
  */
 
 int
 Tcl_WaitForEvent(
-    Tcl_Time *timePtr)		/* Maximum block time, or NULL. */
+    Tcl_Time *timePtr)          /* Maximum block time, or NULL. */
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
     MSG msg;
@@ -440,7 +442,7 @@ Tcl_WaitForEvent(
      */
 
 #ifndef _LANG
-    if (tclStubs.tcl_WaitForEvent != Tcl_WaitForEvent) {
+    if (tclStubs.tcl_WaitForEvent != tclOriginalNotifier.waitForEventProc) {
 	return tclStubs.tcl_WaitForEvent(timePtr);
     }
 #endif
@@ -514,20 +516,20 @@ Tcl_WaitForEvent(
  *
  * Tcl_Sleep --
  *
- *	Delay execution for the specified number of milliseconds.
+ *      Delay execution for the specified number of milliseconds.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Time passes.
+ *      Time passes.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_Sleep(ms)
-    int ms;			/* Number of milliseconds to sleep. */
+    int ms;                     /* Number of milliseconds to sleep. */
 {
     /*
      * Simply calling 'Sleep' for the requisite number of milliseconds
@@ -539,9 +541,9 @@ Tcl_Sleep(ms)
      * as necessary to make the clock advance by the requisite amount.
      */
 
-    Tcl_Time now;		/* Current wall clock time */
-    Tcl_Time desired;		/* Desired wakeup time */
-    int sleepTime = ms;		/* Time to sleep */
+    Tcl_Time now;               /* Current wall clock time */
+    Tcl_Time desired;           /* Desired wakeup time */
+    DWORD sleepTime = ms;       /* Time to sleep */
 
     Tcl_GetTime( &now );
     desired.sec = now.sec + ( ms / 1000 );
