@@ -11,7 +11,7 @@
  *
  * Serial functionality implemented by Rolf.Schroedter@dlr.de
  *
- * RCS: @(#) $Id: tclWinSerial.c,v 1.21 2002/07/19 13:59:10 dkf Exp $
+ * RCS: @(#) $Id: tclWinSerial.c,v 1.25.2.1 2003/05/11 00:31:41 hobbs Exp $
  */
 
 #include "tclWinInt.h"
@@ -103,6 +103,9 @@ typedef struct SerialInfo {
     HANDLE evStartWriter;       /* Auto-reset event used by the main thread to
                                  * signal when the writer thread should attempt
                                  * to write to the serial. */
+    HANDLE evStopWriter;	/* Auto-reset event used by the main thread to
+                                 * signal when the writer thread should close.
+				 */
     DWORD writeError;           /* An error caused by the last background
                                  * write.  Set to 0 if no error has been
                                  * detected.  This word is shared with the
@@ -585,6 +588,7 @@ SerialCloseProc(
     int errorCode, result = 0;
     SerialInfo *infoPtr, **nextPtrPtr;
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    DWORD exitCode;
 
     errorCode = 0;
 
@@ -601,29 +605,53 @@ SerialCloseProc(
          * because it may hang due to handshake
          *    WaitForSingleObject(serialPtr->evWritable, INFINITE);
          */
-        /*
-         * Forcibly terminate the background thread.  We cannot rely on the
-         * thread to cleanly terminate itself because we have no way of
-         * closing the handle without blocking in the case where the
-         * thread is in the middle of an I/O operation.  Note that we need
-         * to guard against terminating the thread while it is in the
-         * middle of Tcl_ThreadAlert because it won't be able to release
-         * the notifier lock.
-         */
 
-        Tcl_MutexLock(&serialMutex);
-        TerminateThread(serialPtr->writeThread, 0);
-        Tcl_MutexUnlock(&serialMutex);
+	/*
+	 * The thread may have already closed on it's own.  Check it's
+	 * exit code.
+	 */
 
-        /*
-         * Wait for the thread to terminate.  This ensures that we are
-         * completely cleaned up before we leave this function.
-         */
+	GetExitCodeThread(serialPtr->writeThread, &exitCode);
+
+	if (exitCode == STILL_ACTIVE) {
+	    /*
+	     * Set the stop event so that if the writer thread is
+	     * blocked in SerialWriterThread on WaitForMultipleEvents, it
+	     * will exit cleanly.
+	     */
+
+	    SetEvent(serialPtr->evStopWriter);
+
+	    /*
+	     * Wait at most 20 milliseconds for the writer thread to
+	     * close.
+	     */
+
+	    if (WaitForSingleObject(serialPtr->writeThread, 20)
+		    == WAIT_TIMEOUT) {
+		/*
+		 * Forcibly terminate the background thread as a last
+		 * resort.  Note that we need to guard against
+		 * terminating the thread while it is in the middle of
+		 * Tcl_ThreadAlert because it won't be able to release
+		 * the notifier lock.
+		 */
+
+		Tcl_MutexLock(&serialMutex);
+
+		/* BUG: this leaks memory */
+		TerminateThread(serialPtr->writeThread, 0);
+
+		Tcl_MutexUnlock(&serialMutex);
+	    }
+	}
 
-        WaitForSingleObject(serialPtr->writeThread, INFINITE);
         CloseHandle(serialPtr->writeThread);
+	CloseHandle(serialPtr->osWrite.hEvent);
+	DeleteCriticalSection(&serialPtr->csWrite);
         CloseHandle(serialPtr->evWritable);
         CloseHandle(serialPtr->evStartWriter);
+        CloseHandle(serialPtr->evStopWriter);
         serialPtr->writeThread = NULL;
 
         PurgeComm(serialPtr->handle, PURGE_TXABORT | PURGE_TXCLEAR);
@@ -632,18 +660,18 @@ SerialCloseProc(
 
     /*
      * Don't close the Win32 handle if the handle is a standard channel
-     * during the exit process.  Otherwise, one thread may kill the stdio
-     * of another.
+     * during the thread exit process.  Otherwise, one thread may kill
+     * the stdio of another.
      */
 
-    if (!TclInExit()
-        || ((GetStdHandle(STD_INPUT_HANDLE) != serialPtr->handle)
-        && (GetStdHandle(STD_OUTPUT_HANDLE) != serialPtr->handle)
-        && (GetStdHandle(STD_ERROR_HANDLE) != serialPtr->handle))) {
-           if (CloseHandle(serialPtr->handle) == FALSE) {
-                TclWinConvertError(GetLastError());
-                errorCode = errno;
-            }
+    if (!TclInThreadExit()
+	|| ((GetStdHandle(STD_INPUT_HANDLE) != serialPtr->handle)
+	&& (GetStdHandle(STD_OUTPUT_HANDLE) != serialPtr->handle)
+	&& (GetStdHandle(STD_ERROR_HANDLE) != serialPtr->handle))) {
+	    if (CloseHandle(serialPtr->handle) == FALSE) {
+		TclWinConvertError(GetLastError());
+		errorCode = errno;
+	    }
     }
 
     serialPtr->watchMask &= serialPtr->validMask;
@@ -653,8 +681,8 @@ SerialCloseProc(
      */
 
     for (nextPtrPtr = &(tsdPtr->firstSerialPtr), infoPtr = *nextPtrPtr;
-            infoPtr != NULL;
-                    nextPtrPtr = &infoPtr->nextPtr, infoPtr = *nextPtrPtr) {
+	    infoPtr != NULL;
+	    nextPtrPtr = &infoPtr->nextPtr, infoPtr = *nextPtrPtr) {
         if (infoPtr == (SerialInfo *)serialPtr) {
             *nextPtrPtr = infoPtr->nextPtr;
             break;
@@ -703,20 +731,20 @@ blockingRead(
     LPOVERLAPPED osPtr )    /* OVERLAPPED structure */
 {
     /*
-    *  Perform overlapped blocking read.
-    *  1. Reset the overlapped event
-    *  2. Start overlapped read operation
-    *  3. Wait for completion
-    */
+     *  Perform overlapped blocking read.
+     *  1. Reset the overlapped event
+     *  2. Start overlapped read operation
+     *  3. Wait for completion
+     */
 
-	/*
-	* Set Offset to ZERO, otherwise NT4.0 may report an error
-	*/
-	osPtr->Offset = osPtr->OffsetHigh = 0;
+    /*
+     * Set Offset to ZERO, otherwise NT4.0 may report an error.
+     */
+    osPtr->Offset = osPtr->OffsetHigh = 0;
     ResetEvent(osPtr->hEvent);
     if (! ReadFile(infoPtr->handle, buf, bufSize, lpRead, osPtr) ) {
         if (GetLastError() != ERROR_IO_PENDING) {
-            /* ReadFile failed, but it isn't delayed. Report error */
+            /* ReadFile failed, but it isn't delayed. Report error. */
             return FALSE;
         } else {
             /* Read is pending, wait for completion, timeout ? */
@@ -937,7 +965,7 @@ SerialOutputProc(
     int *errorCode)             /* Where to store error code. */
 {
     SerialInfo *infoPtr = (SerialInfo *) instanceData;
-    int bytesWritten, timeout;
+    DWORD bytesWritten, timeout;
 
     *errorCode = 0;
 
@@ -1002,13 +1030,13 @@ SerialOutputProc(
                 ckfree(infoPtr->writeBuf);
             }
             infoPtr->writeBufLen = toWrite;
-            infoPtr->writeBuf = ckalloc(toWrite);
+            infoPtr->writeBuf = ckalloc((unsigned int) toWrite);
         }
-        memcpy(infoPtr->writeBuf, buf, toWrite);
+        memcpy(infoPtr->writeBuf, buf, (size_t) toWrite);
         infoPtr->toWrite = toWrite;
         ResetEvent(infoPtr->evWritable);
         SetEvent(infoPtr->evStartWriter);
-        bytesWritten = toWrite;
+        bytesWritten = (DWORD) toWrite;
 
     } else {
         /*
@@ -1019,7 +1047,7 @@ SerialOutputProc(
                 &bytesWritten, &infoPtr->osWrite) ) {
             goto writeError;
         }
-        if (bytesWritten != toWrite) {
+        if (bytesWritten != (DWORD) toWrite) {
             /* Write timeout */
             infoPtr->lastError |= CE_PTO;
             errno = EIO;
@@ -1027,7 +1055,7 @@ SerialOutputProc(
         }
     }
 
-    return bytesWritten;
+    return (int) bytesWritten;
 
 writeError:
     TclWinConvertError(GetLastError());
@@ -1246,17 +1274,32 @@ SerialWriterThread(LPVOID arg)
 {
 
     SerialInfo *infoPtr = (SerialInfo *)arg;
-    HANDLE *handle = infoPtr->handle;
-    DWORD bytesWritten, toWrite;
+    DWORD bytesWritten, toWrite, waitResult;
     char *buf;
     OVERLAPPED myWrite; /* have an own OVERLAPPED in this thread */
+    HANDLE wEvents[2];
+
+    /*
+     * The stop event takes precedence by being first in the list.
+     */
+    wEvents[0] = infoPtr->evStopWriter;
+    wEvents[1] = infoPtr->evStartWriter;
 
     for (;;) {
         /*
          * Wait for the main thread to signal before attempting to write.
          */
 
-        WaitForSingleObject(infoPtr->evStartWriter, INFINITE);
+	waitResult = WaitForMultipleObjects(2, wEvents, FALSE, INFINITE);
+
+	if (waitResult != (WAIT_OBJECT_0 + 1)) {
+	    /*
+	     * The start event was not signaled.  It might be the stop event
+	     * or an error, so exit.
+	     */
+
+	    break;
+	}
 
         buf = infoPtr->writeBuf;
         toWrite = infoPtr->toWrite;
@@ -1306,7 +1349,8 @@ SerialWriterThread(LPVOID arg)
         Tcl_ThreadAlert(infoPtr->threadId);
         Tcl_MutexUnlock(&serialMutex);
     }
-    return 0;                   /* NOT REACHED */
+
+    return 0;
 }
 
 
@@ -1426,8 +1470,9 @@ TclWinOpenSerialChannel(handle, channelName, permissions)
         infoPtr->osWrite.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
         infoPtr->evWritable = CreateEvent(NULL, TRUE, TRUE, NULL);
         infoPtr->evStartWriter = CreateEvent(NULL, FALSE, FALSE, NULL);
+	infoPtr->evStopWriter = CreateEvent(NULL, FALSE, FALSE, NULL);
         InitializeCriticalSection(&infoPtr->csWrite);
-        infoPtr->writeThread = CreateThread(NULL, 8000, SerialWriterThread,
+        infoPtr->writeThread = CreateThread(NULL, 256, SerialWriterThread,
             infoPtr, 0, &id);
     }
 
@@ -1477,7 +1522,7 @@ SerialErrorStr(error, dsPtr)
     if( (error & CE_PTO) != 0) {    /* PTO used to signal WRITE-TIMEOUT */
                 Tcl_DStringAppendElement(dsPtr, "TIMEOUT");
     }
-    if( (error & ~(SERIAL_READ_ERRORS | SERIAL_WRITE_ERRORS)) != 0) {
+    if( (error & ~((DWORD) (SERIAL_READ_ERRORS | SERIAL_WRITE_ERRORS))) != 0) {
                 char buf[TCL_INTEGER_SPACE + 1];
                 wsprintfA(buf, "%d", error);
                 Tcl_DStringAppendElement(dsPtr, buf);
@@ -1537,7 +1582,7 @@ SerialSetOptionProc(instanceData, interp, optionName, value)
     Tcl_DString ds;
     CONST TCHAR *native;
     int argc;
-    char **argv;
+    CONST char **argv;
 
     infoPtr = (SerialInfo *) instanceData;
 
@@ -1705,7 +1750,8 @@ SerialSetOptionProc(instanceData, interp, optionName, value)
                 return TCL_ERROR;
             }
             if (strnicmp(argv[0], "DTR", strlen(argv[0])) == 0) {
-                if (! EscapeCommFunction(infoPtr->handle, flag ? SETDTR : CLRDTR)) {
+                if (! EscapeCommFunction(infoPtr->handle, flag ?
+                        (DWORD) SETDTR : (DWORD) CLRDTR)) {
                     if (interp) {
                         Tcl_AppendResult(interp,
                             "can't set DTR signal", (char *) NULL);
@@ -1713,7 +1759,8 @@ SerialSetOptionProc(instanceData, interp, optionName, value)
                     return TCL_ERROR;
                 }
             } else if (strnicmp(argv[0], "RTS", strlen(argv[0])) == 0) {
-                if (! EscapeCommFunction(infoPtr->handle, flag ? SETRTS : CLRRTS)) {
+                if (! EscapeCommFunction(infoPtr->handle, flag ?
+                        (DWORD) SETRTS : (DWORD) CLRRTS)) {
                     if (interp) {
                         Tcl_AppendResult(interp,
                             "can't set RTS signal", (char *) NULL);
@@ -1721,7 +1768,8 @@ SerialSetOptionProc(instanceData, interp, optionName, value)
                     return TCL_ERROR;
                 }
             } else if (strnicmp(argv[0], "BREAK", strlen(argv[0])) == 0) {
-                if (! EscapeCommFunction(infoPtr->handle, flag ? SETBREAK : CLRBREAK)) {
+                if (! EscapeCommFunction(infoPtr->handle, flag ?
+                        (DWORD) SETBREAK : (DWORD) CLRBREAK)) {
                     if (interp) {
                         Tcl_AppendResult(interp,
                             "can't set BREAK signal", (char *) NULL);
@@ -1751,7 +1799,7 @@ SerialSetOptionProc(instanceData, interp, optionName, value)
         /*
         * -sysbuffer 4096 or -sysbuffer {64536 4096}
         */
-        size_t inSize = -1, outSize = -1;
+        size_t inSize = (size_t) -1, outSize = (size_t) -1;
 
         if (Tcl_SplitList(interp, value, &argc, &argv) == TCL_ERROR) {
             return TCL_ERROR;
@@ -2004,7 +2052,7 @@ SerialGetOptionProc(instanceData, interp, optionName, dsPtr)
     if ((len > 1) && (strncmp(optionName, "-queue", len) == 0)) {
         char buf[TCL_INTEGER_SPACE + 1];
         COMSTAT cStat;
-        int error;
+        DWORD error;
 	int inBuffered, outBuffered, count;
 
         valid = 1;
