@@ -94,7 +94,7 @@ static Tk_ConfigSpec imageTextItemConfigSpecs[] = {
 #define DEF_IMAGETEXTSTYLE_DISABLED_BG_COLOR	DISABLED_BG
 #define DEF_IMAGETEXTSTYLE_DISABLED_BG_MONO	WHITE
 
-#define DEF_IMAGETEXTSTYLE_FONT	 "-Adobe-Helvetica-Bold-R-Normal--*-120-*"
+#define DEF_IMAGETEXTSTYLE_FONT		CTL_FONT
 #define DEF_IMAGETEXTSTYLE_GAP		"4"
 #define DEF_IMAGETEXTSTYLE_PADX		"2"
 #define DEF_IMAGETEXTSTYLE_PADY		"2"
@@ -396,7 +396,7 @@ static void Tix_ImageTextItemDisplay(pixmap, gc, iPtr, x, y,
     }
 
     TixGetColorDItemGC(iPtr, &backGC, &foreGC, flags);
-    TixpStartSubRegionDraw(itPtr->ddPtr->display, pixmap, foreGC,
+    TixpStartSubRegionDraw(itPtr->ddPtr, pixmap, foreGC,
 	    &subReg, 0, 0, x, y, width, height,
 	    itPtr->size[0], itPtr->size[1]);
 
