@@ -79,7 +79,7 @@ static Tk_ConfigSpec textItemConfigSpecs[] = {
 
 #define DEF_TEXTSTYLE_PADX			"2"
 #define DEF_TEXTSTYLE_PADY			"2"
-#define DEF_TEXTSTYLE_FONT	     "-Adobe-Helvetica-Bold-R-Normal--*-120-*"
+#define DEF_TEXTSTYLE_FONT			"Helvetica -12 bold"
 #define DEF_TEXTSTYLE_JUSTIFY			"left"
 #define DEF_TEXTSTYLE_WLENGTH			"0"
 #define DEF_TEXTSTYLE_ANCHOR			"w"
@@ -353,7 +353,7 @@ static void Tix_TextItemDisplay(pixmap, gc, iPtr, x, y, width, height, flags)
 
     TixGetColorDItemGC(iPtr, &backGC, &foreGC, flags);
 
-    TixpStartSubRegionDraw(itPtr->ddPtr->display, pixmap, foreGC,
+    TixpStartSubRegionDraw(itPtr->ddPtr, pixmap, foreGC,
 	    &subReg, 0, 0, x, y, width, height,
 	    itPtr->size[0], itPtr->size[1]);
 
@@ -369,10 +369,18 @@ static void Tix_TextItemDisplay(pixmap, gc, iPtr, x, y, width, height, flags)
 	x += itPtr->stylePtr->pad[0];
 	y += itPtr->stylePtr->pad[1];
 
+        /* FIXME?
+           A. is numChars still valid and is it bytes or chars ?
+           B. If width < itPtr->textW this falls out of the
+              region when using Xft to draw chars
+           C. What do we want to happen for width < textW anyway?
+               - clip (as before) or wrap ?
+         */
+
 	TixpSubRegDisplayText(itPtr->ddPtr->display, pixmap, foreGC,
 		&subReg, itPtr->stylePtr->font, Tcl_GetString(itPtr->text),
-		itPtr->numChars, x, y, itPtr->textW, itPtr->stylePtr->justify,
-		itPtr->underline);
+		itPtr->numChars, x, y,
+		width, itPtr->stylePtr->justify, itPtr->underline);
     }
 
     TixpEndSubRegionDraw(itPtr->ddPtr->display, pixmap, foreGC,
