@@ -883,7 +883,7 @@ IsMwmRunning(interp, wmPtr)
 	Window	top, parent, *children;
 	unsigned int num_children;
 	int	returnVal = 0;
-	int	i;
+	unsigned	i;
 
 	if (XQueryTree(Tk_Display(wmPtr->tkwin), root, &top, &parent,
 	    &children, &num_children)) {
