@@ -143,10 +143,6 @@
 #  define TixpEndSubRegionDraw (*TixintVptr->V_TixpEndSubRegionDraw)
 #endif
 
-#ifndef TixpSetWindowParent
-#  define TixpSetWindowParent (*TixintVptr->V_TixpSetWindowParent)
-#endif
-
 #ifndef TixpStartSubRegionDraw
 #  define TixpStartSubRegionDraw (*TixintVptr->V_TixpStartSubRegionDraw)
 #endif
@@ -167,5 +163,9 @@
 #  define TixpSubRegFillRectangle (*TixintVptr->V_TixpSubRegFillRectangle)
 #endif
 
+#ifndef tixStrDup
+#  define tixStrDup (*TixintVptr->V_tixStrDup)
+#endif
+
 #endif /* NO_VTABLES */
 #endif /* _TIXINT_VM */
