@@ -1,7 +1,9 @@
 #include "tk.h"
 #include "tk_f.h"
+static unsigned TkVSize(void) { return sizeof(TkVtab);}
 static TkVtab TkVtable =
 {
+ TkVSize,
 #define VFUNC(type,name,mem,args) name,
 #define VVAR(type,name,mem)      &name,
 #include "tk.t"
